package be.iminds.ilabt.jfed.restapi.jfedclient;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experiment.ExperimentController;
import be.iminds.ilabt.jfed.experiment.ExperimentState;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.TestbedInfoSource;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.restapi.debugdata.ClientApiCallDDCollector;
import be.iminds.ilabt.jfed.restapi.json.ExperimentTestCollection;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.JFedTrustStore;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * MockJFedExperimenterApi: this pretends to be a JFedExperimenterApi
 * it uses ExperimenterTestCollection to mock most functionality
 */
public class MockJFedExperimenterApi implements JFedExperimenterApi {
    private final ExperimentTestCollection experimentTestCollection;
    private be.iminds.ilabt.jfed.restapi.json.Experiment activeExperiment;

    public MockJFedExperimenterApi() throws Exception {
        this.experimentTestCollection = new ExperimentTestCollection();
        this.activeExperiment = null;
    }

    @Override
    public void createSliceAndSlivers(String sliceName, String project, String rspec, Instant expire) {
        this.activeExperiment = experimentTestCollection.getBySliceName(sliceName);
    }
    @Override
    public void recoverSlice(GeniUrn sliceUrn) {
        this.activeExperiment = experimentTestCollection.getBySliceUrn(sliceUrn);
    }




    @Override
    public Logger getLogger() {
        return mock(Logger.class);
    }

    @Override
    public GeniUser getUser() {
        return mock(GeniUser.class);
    }

    @Override
    public TestbedInfoSource getTestbedInfoSource() {
        return mock(TestbedInfoSource.class);
    }

    @Override
    public AuthorityFinder getAuthorityFinder() {
        return mock(AuthorityFinder.class);
    }

    @Override
    public JFedTrustStore getJFedTrustStore() {
        return mock(JFedTrustStore.class);
    }

    @Override public boolean removeClientApiCallDDCollector(ClientApiCallDDCollector clientApiCallDDCollector) { return false; }
    @Override public void addClientApiCallDDCollector(ClientApiCallDDCollector clientApiCallDDCollector) { }
    @Override public void setClientApiCallDDCollector(ClientApiCallDDCollector clientApiCallDDCollector) { }

    @Override
    public List<GeniUrn> getProjectsForUser(GeniUrn user) {
        return Arrays.asList(GeniUrn.parse("urn:publicid:IDN+wall2.ilabt.iminds.be+project+test"));
    }

    @Override
    public List<GeniUrn> getSlicesForUser(GeniUrn user) {
        return Arrays.asList(GeniUrn.parse("urn:publicid:IDN+wall2.ilabt.iminds.be:test+slice+slice1"));
    }


    @Override
    public void refreshStatusIfNeeded() {

    }

    @Override
    public void renewSlice(Timestamp newExpire) {

    }

    @Override
    public void deleteSlice() {

    }

    @Override
    public String getProject() {
        return activeExperiment.getProject();
    }

    @Override
    public String getSliceName() {
        return activeExperiment.getSliceName();
    }

    @Override
    public GeniUrn getSliceUrn() {
        return activeExperiment.getSliceUrn();
    }

    @Override
    public Timestamp getExpire() {
        return activeExperiment.getExpire();
    }

    @Override
    public String getManifestRspec() {
        return activeExperiment.getManifestRspec();
    }

    @Override
    public Slice getExperimentSlice() {
        return mock(Slice.class);
    }

    @Override
    public StatusDetails getExperimentSliceStatus() {
        switch (activeExperiment.getStatus()) {
            case FAILED: return new StatusDetails("FAIL");
            case NONEXISTING: return new StatusDetails("UNALLOCATED"); //UNINITIALISED? //UNKNOWN?
            case READY: return new StatusDetails("READY");
            case STARTING: return new StatusDetails("CHANGING");
            default: throw new RuntimeException("Unhandled status: "+activeExperiment.getStatus());
        }
    }

    @Override
    public Experiment getExperiment() {
        Experiment res = mock(Experiment.class);
        when(res.getSlice()).thenAnswer((m) -> getExperimentSlice());
        when(res.getName()).thenAnswer((m) -> activeExperiment.getSliceName());
        return res;
    }

    @Override
    public ExperimentState getExperimentState() {
        switch (activeExperiment.getStatus()) {
            case FAILED: return ExperimentState.FAILED;
            case NONEXISTING: return ExperimentState.EMPTY;
            case READY: return ExperimentState.READY;
            case STARTING: return ExperimentState.PROVISIONING;
            default: throw new RuntimeException("Unhandled status: "+activeExperiment.getStatus());
        }
    }

    @Override
    public ExperimentController getExperimentController() {
        return mock(ExperimentController.class);
    }
}
